/* ================================================================================== */
/* Copyright (c) 1998-1999 3Com Corporation or its subsidiaries. All rights reserved. */
/* ================================================================================== */

#ifndef _BYTESWAPPING_H_
#define _BYTESWAPPING_H_

#if !defined (BYTESWAP)
	#if EMULATION_LEVEL == EMULATION_WINDOWS
		#define BYTESWAP 1
		#define WORDSWAP_MEMORY 1
	#elif defined (UNIX)
		#include <endian.h>
		#if __BYTE_ORDER == __LITTLE_ENDIAN
			#define BYTESWAP			1
			#define WORDSWAP_MEMORY		1
		#endif
	#endif
#endif


/*
**	--------------------------------------------------------------------------------
**		Byte swapping
**	--------------------------------------------------------------------------------
*/

#define BYTE_SWAP_16(n)	((((unsigned short) n) >> 8) |	\
						 (((unsigned short) n) << 8))

#define BYTE_SWAP_32(n)	( (((unsigned long) n) << 24) |					\
						 ((((unsigned long) n) << 8) & 0x00FF0000) |	\
						 ((((unsigned long) n) >> 8) & 0x0000FF00) |	\
						  (((unsigned long) n) >> 24))


#if BYTESWAP

	inline void Canonical(bool&)
	{
	}

	inline void Canonical(char&)
	{
	}

	inline void Canonical(signed char&)
	{
	}

	inline void Canonical(unsigned char&)
	{
	}

	inline void Canonical(short& v)
	{
		v = (short) BYTE_SWAP_16(v);
	}

	inline void Canonical(unsigned short& v)
	{
		v = (unsigned short) BYTE_SWAP_16(v);
	}

	inline void Canonical(int& v)
	{
		if (sizeof(int) == 4)
			v = (int) BYTE_SWAP_32(v);

		else if (sizeof(int) == 2)
			v = (int) BYTE_SWAP_16(v);
	}

	inline void Canonical(unsigned int& v)
	{
		if (sizeof(unsigned int) == 4)
			v = (unsigned int) BYTE_SWAP_32(v);

		else if (sizeof(unsigned int) == 2)
			v = (unsigned int) BYTE_SWAP_16(v);
	}

	inline void Canonical(long& v)
	{
		v = (long) BYTE_SWAP_32(v);
	}

	inline void Canonical(unsigned long& v)
	{
		v = (unsigned long) BYTE_SWAP_32(v);
	}

	template <class T>
	inline void Canonical(T*& v)
	{
		v = (T*) BYTE_SWAP_32(v);
	}

	inline void Canonical(WinHandle& v)
	{
		v = (WinHandle) BYTE_SWAP_32(v);
	}

	inline void Canonical(SysKernelInfoSelector& v)
	{
		if (sizeof(SysKernelInfoSelector) == 4)
			v = (SysKernelInfoSelector) BYTE_SWAP_32(v);

		else if (sizeof(SysKernelInfoSelector) == 2)
			v = (SysKernelInfoSelector) BYTE_SWAP_16(v);
	}

	inline void Canonical(enum events& v)
	{
		if (sizeof(enum events) == 4)
			v = (enum events) BYTE_SWAP_32(v);

		else if (sizeof(enum events) == 2)
			v = (enum events) BYTE_SWAP_16(v);
	}

	void Canonical (BreakpointType& p);
	void Canonical (CardHeaderType& p);
	void Canonical (DatabaseDirType& p);
	void Canonical (DatabaseHdrType& p);
	void Canonical (DmAccessType& p);
	void Canonical (DmOpenInfoType& p);
	void Canonical (DmSearchStateType& p);
	void Canonical (EventType& event);
	void Canonical (FieldAttrType& p);
	void Canonical (HwrDBallEZType&);
	void Canonical (HwrDBallType&);
	void Canonical (M68KRegsType& p);
	void Canonical (Mem1ChunkHeaderType& p);
	void Canonical (Mem1HeapHeaderType& p);
	void Canonical (Mem1MstrPtrTableType& p);
	void Canonical (Mem2HeapHeaderType& p);
	void Canonical (MemChunkHeaderType& p);
	void Canonical (MemHeapHeaderType& p);
	void Canonical (MemMstrPtrTableType& p);
	void Canonical (PenBtnInfoType& p);
	void Canonical (Point& p);
	void Canonical (PointType& p);
	void Canonical (RecordListType& p);
	void Canonical (RecordEntryType& p);
	void Canonical (RectangleType& p);
	void Canonical (RsrcEntryType& p);
	void Canonical (SlkPktHeaderType& p);
	void Canonical (SlkPktFooterType& p);
	void Canonical (SysAppInfoType& p);
	void Canonical (SysKernelInfoType& p);
	void Canonical (SysNVParamsType& p);
	void Canonical (SysPktBodyType& p);
	void Canonical (SysPktChecksumType& p);		// --- response is same type
	void Canonical (SysPktCommCmdType& p);
	void Canonical (SysPktCommRspType& p);
	void Canonical (SysPktContinueCmdType& p);	// --- no response
	void Canonical (SysPktDbgBreakToggleCmdType& p);
	void Canonical (SysPktDbgBreakToggleRspType& p);
	void Canonical (sysPktExecFlashType& p);	// --- no response
	void Canonical (SysPktFindCmdType& p);
	void Canonical (SysPktFindRspType& p);
	void Canonical (SysPktFlashWriteType& p);	// --- response is same type
	void Canonical (SysPktGetBreakpointsCmdType& p);
	void Canonical (SysPktGetBreakpointsRspType& p);
	void Canonical (SysPktGetTrapBreaksCmdType& p);
	void Canonical (SysPktGetTrapBreaksRspType& p);
	void Canonical (SysPktGetTrapConditionsCmdType& p);
	void Canonical (SysPktGetTrapConditionsRspType& p);
	void Canonical (SysPktGremlinsCmdType&);	// --- no response
	void Canonical (SysPktReadMemCmdType& p);
	void Canonical (SysPktReadMemRspType& p);
	void Canonical (SysPktReadRegsCmdType& p);
	void Canonical (SysPktReadRegsRspType& p);
	void Canonical (SysPktRemoteMsgCmdType& p);	// --- no response
	void Canonical (SysPktRPCType& p);			// --- response is same type
	void Canonical (SysPktRtnNameCmdType& p);
	void Canonical (SysPktRtnNameRspType& p);
	void Canonical (SysPktSetBreakpointsCmdType& p);
	void Canonical (SysPktSetBreakpointsRspType& p);
	void Canonical (SysPktSetTrapBreaksCmdType& p);
	void Canonical (SysPktSetTrapBreaksRspType& p);
	void Canonical (SysPktSetTrapConditionsCmdType& p);
	void Canonical (SysPktSetTrapConditionsRspType& p);
	void Canonical (SysPktStateCmdType& p);
	void Canonical (SysPktStateRspType& p);
	void Canonical (SysPktWriteRegsCmdType& p);
	void Canonical (SysPktWriteRegsRspType& p);
	void Canonical (SysPktWriteMemCmdType& p);
	void Canonical (SysPktWriteMemRspType& p);

	void Canonical (regstruct& r);
	void Canonical (struct flag_struct& r);


#else	// !BYTESWAP

	template <class T>
	inline void Canonical(T&)
	{
	}

#endif	// BYTESWAP

#if WORDSWAP_MEMORY

	inline void ByteswapWords (void* start, unsigned long length)
	{
		for (unsigned long ii = 0; ii < length / 2; ++ii)
		{
			unsigned short*	p = ((unsigned short*) start) + ii;
			unsigned char*	bp = (unsigned char*) p;
			*p = (((unsigned short) *bp) << 8) | (unsigned short) (*(bp + 1));
		}
	}

#else

	inline void ByteswapWords (void*, unsigned long)
	{
	}

#endif	// WORDSWAP_MEMORY

#endif /* _BYTESWAPPING_H_ */

