/* ================================================================================== */
/* Copyright (c) 1998-1999 3Com Corporation or its subsidiaries. All rights reserved. */
/* ================================================================================== */

#ifndef _MISCELLANEOUS_H_
#define _MISCELLANEOUS_H_

#include "Byteswapping.h"		// Canonical

class StMemory
{
	public:
		StMemory		(	char*	inPtr = NULL);

		StMemory		(	long	inSize,
							Bool	inClearBytes = false);

		~StMemory		();

		operator char*	()			{ return mPtr; }

		char*	Get		() const	{ return mPtr; }
		Bool	IsOwner	() const	{ return mIsOwner; }
		Bool	IsValid	() const	{ return (mPtr != NULL); }

		void	Adopt	(	char*	inPtr);
		char*	Release	() const;
		void	Dispose	();

	protected:
		char*			mPtr;
		mutable Bool	mIsOwner;

	private:
		StMemory	( const StMemory &inPointerBlock);
		StMemory&	operator = (const StMemory &inPointerBlock);
};


class StMemoryMapper
{
	public:
		StMemoryMapper	(const void* memory, long size);
		~StMemoryMapper	(void);

	private:
		const void*	fMemory;
};

template <class T>
class StCanonical
{
	public:
		StCanonical (T& obj) :
			fObject (obj)
		{
			Canonical (fObject);
		}

		~StCanonical ()
		{
			Canonical (fObject);
		}

	private:
		T&	fObject;
};

class StWordSwapper
{
	public:
		StWordSwapper (void* memory, long length) :
			fMemory (memory),
			fLength (length)
		{
			::ByteswapWords (fMemory, fLength);
		}

		~StWordSwapper ()
		{
			::ByteswapWords (fMemory, fLength);
		}

	private:
		void*	fMemory;
		long	fLength;
};

struct AppInfo
{
	ULong		creator;
	ULong		type;
	UInt		version;
	LocalID		dbID;
	UInt 		cardNo;
	ULong		modDate;
	Word		dbAttrs;

	char		name[dmDBNameLength];

	Bool operator < (const AppInfo& other) const
		{ if (creator < other.creator) return true;
		  if (creator > other.creator) return false;
		  if (type < other.type) return true;
		  return false; }

	Bool operator == (const AppInfo& other) const
		{ return (creator == other.creator) &&
				 (type == other.type); }
};

typedef vector<AppInfo>	AppInfoList;

#ifdef __MSL_FIX_ITERATORS__
__MSL_FIX_ITERATORS__(AppInfo)
#endif

// Inline function to turn a trap word (0xA###) into an index into the
// trap table.  The method used here (masking off the uppermost nybble
// instead of, say, subtracting sysTrapBase) matches the ROM.

inline uae_u16 SysTrapIndex (uae_u16 trapWord)
{
	return (uae_u16) (trapWord & ~0xF000);
}

inline uae_u16 LibTrapIndex (uae_u16 trapWord)
{
	return (uae_u16) (SysTrapIndex (trapWord) - SysTrapIndex (sysLibTrapBase));
}

inline Bool IsSystemTrap (uae_u16 trapWord)
{
	return SysTrapIndex (trapWord) < SysTrapIndex (sysLibTrapBase);
}

inline Bool IsLibraryTrap (uae_u16 trapWord)
{
	return !IsSystemTrap (trapWord);
}

enum DeviceType 
{
	kDeviceUnspecified	= 0,
	kDevicePilot1000,
	kDevicePilot5000,
	kDevicePalmPilotPersonal,
	kDevicePalmPilotProfessional,
	kDevicePalmPilotWithUpgrade,
	kDevicePalmIII,
	kDevicePalmVII,
	kDeviceUnused1,
	kDeviceQualComm,
	kDevicePalmV,
	kDevicePalmIIIx,
	kDeviceUnused2,
	kDeviceLast
};

enum SilkscreenType
{
	kSilkscreenUnspecified,
	kSilkscreenEnglish,
	kSilkscreenJapanese,
	kSilkscreenLast
};

struct SystemCallContext
{
	uaecptr		fPC;			// PC at which the system call was made.
	uaecptr		fNextPC;		// PC of instruction after system call.
	uaecptr		fDestPC;		// Address of system function.
	uae_u16		fTrapWord;		// Trapword, even for SYSTRAP_FAST calls.
	uae_u16		fTrapIndex;		// Trapword with the high 4 bits removed.
	uaecptr		fExtra;			// RefNum for library calls, D2 for "dispatch" calls.
};

Bool	StartsWith				(const char* s, const char* pattern);
Bool	EndsWith				(const char* s, const char* pattern);

void	GetApplicationNames		(AppInfoList& appList);

Bool	IsExecutable			(DWord dbType, DWord dbCreator, Word dbAttrs);
Bool	IsVisible				(DWord dbType, DWord dbCreator, Word dbAttrs);

void	AutoSaveState			(void);

void	ResetCalibrationInfo	(void);
void	SetHotSyncUserName		(const char*);

void	RunLengthEncode			(void** srcPP, void** dstPP, long srcBytes, long dstBytes);
void	RunLengthDecode			(void** srcPP, void** dstPP, long srcBytes, long dstBytes);
long	RunLengthWorstSize		(long);

void	GzipEncode				(void** srcPP, void** dstPP, long srcBytes, long dstBytes);
void	GzipDecode				(void** srcPP, void** dstPP, long srcBytes, long dstBytes);
long	GzipWorstSize			(long);

uae_u32	NextPowerOf2			(uae_u32 x);
uae_u32	DateToDays				(uae_u32 year, uae_u32 month, uae_u32 day);

string	GetLibraryName			(uae_u16 refNum);

uaecptr GetFunctionAddress		(uae_u16 trapWord, uae_u32 extra = sysInvalidRefNum, Bool digDeep = false);
uaecptr GetLibFunctionAddress	(uae_u16 trapWord, Word refNum, Bool digDeep);
uaecptr GetSysFunctionAddress	(uae_u16 trapWord, uae_u32 extra, Bool digDeep);
uaecptr GetStdDispatchAddress	(uaecptr entryPt, uae_u32 regD2);
uaecptr GetFlpEmDispatchAddress	(uaecptr entryPt, uae_u32 regD2);

extern uae_u32	gLibErrorBase;

char*	GetTrapName				(const SystemCallContext&, Bool digDeep = false);
char*	GetTrapName				(uae_u16 trapWord, uae_u32 extra = sysInvalidRefNum, Bool digDeep = false);
void	FindTrapName			(uae_u16 trapWord, char* nameP, uae_u32 extra = sysInvalidRefNum, Bool digDeep = false);

void	FindFunctionName		(uaecptr addr,
								 char* nameP,
								 uaecptr* startAddrP = NULL,
								 uaecptr* endAddrP = NULL);
uaecptr	FindFunctionStart		(uaecptr addr);
uaecptr	FindFunctionEnd			(uaecptr addr);
void	CopyMacsbugName			(uaecptr eof, char* name);
Bool	EndOfFunctionSequence	(uaecptr addr);
uae_u8	MacsbugNameLength		(uaecptr& addr, Bool* isFixed);
Bool	ValidMacsbugChar		(uae_u8 ch);

Bool	GetSystemCallContext	(uaecptr, SystemCallContext&);

void	GetHostTime				(long* hour, long* min, long* sec);
void	GetHostDate				(long* year, long* month, long* day);

const char*	LaunchCmdToString	(Word cmd);

#endif	// _MISCELLANEOUS_H_
