/*********************************************************************
 *
 * Copyright (c) 1998
 * 3Com/Palm Computing Division.  All rights reserved.
 *
 * Portions copyright (c) 1996-1997
 * Greg Hewgill.  All rights reserved.
 *
 * Portions copyright (c) 1995-1997
 * Illume Software, Inc.  All rights reserved.
 *
 * Portions copyright (c) 1995-1997
 * Bernd Schmidt.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *		This product includes software developed by 3Com and its
 *		contributors.
 *
 * 4. Neither 3Com nor the names of its contributors may be used to
 *    endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE 3COM AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 3COM OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 *******************************************************************/


// -------------------------------------------------------------
// Include Header files. When building the PalmOSEmulator, we 
//  only use a subset of all the PalmOS Headers
// -------------------------------------------------------------
#if BUILDING_EMULATOR

  #include "Switches.h"
  #include "UAE.h"					// uae_u32

  #include "Palm.h"					// PointType, RectangleType, etc.
  #include "DebugMgr.h"				// SysPktReadRegsRspType2, etc.

  #define	NON_PORTABLE
  #include "M68328Hwr.h"			// HwrDBallType

	  // CPU_REG.cpp includes both this header and Bank_MC68328.h. These
	  // headers include M68EZ328Hwr.h and M68328Hwr.h each, which define
	  // conflicting definitions of HwrDBallType and HwrDBallPtr. To
	  // avoid that collision, use some pre-processor majik.
  #define HwrDBallType	HwrDBallEZType
  #define HwrDBallPtr	HwrDBallEZPtr
  #include "M68EZ328Hwr.h"		// HwrDBallType
  #undef HwrDBallType
  #undef HwrDBallPtr

// -------------------------------------------------------------
// Other apps (i.e. PalmDebugger) use the full set. 
// -------------------------------------------------------------
#else

  #include <Pilot.h>
  #define	NON_PORTABLE
  #include <AlarmPrv.h>
  #include <SystemPrv.h>
  #include <DataPrv.h>

#endif // !BUILDING_EMULATOR



#if CPU_TYPE == CPU_x86
  #define CANONICAL(param) Canonical(param)
#else
  #define CANONICAL(param) 
#endif


/*
**	--------------------------------------------------------------------------------
**		Byte swapping
**	--------------------------------------------------------------------------------
*/

#define BYTE_SWAP_16(n)	((((Word) n) >> 8) |	\
						 (((Word) n) << 8))

#define BYTE_SWAP_32(n)	( (((DWord) n) << 24) |					\
						 ((((DWord) n) << 8) & 0x00FF0000) |	\
						 ((((DWord) n) >> 8) & 0x0000FF00) |	\
						  (((DWord) n) >> 24))


	inline void Canonical(char& /*v*/)
	{
	}

	inline void Canonical(signed char& /*v*/)
	{
	}

	inline void Canonical(unsigned char& /*v*/)
	{
	}

	inline void Canonical(short& v)
	{
		v = (short) BYTE_SWAP_16(v);
	}

	inline void Canonical(unsigned short& v)
	{
		v = (unsigned short) BYTE_SWAP_16(v);
	}

	inline void Canonical(int& v)
	{
		if (sizeof(int) == 4)
			v = (int) BYTE_SWAP_32(v);

		else if (sizeof(int) == 2)
			v = (int) BYTE_SWAP_16(v);
	}

	inline void Canonical(unsigned int& v)
	{
		if (sizeof(unsigned int) == 4)
			v = (unsigned int) BYTE_SWAP_32(v);

		else if (sizeof(unsigned int) == 2)
			v = (unsigned int) BYTE_SWAP_16(v);
	}

	inline void Canonical(long& v)
	{
		v = (long) BYTE_SWAP_32(v);
	}

	inline void Canonical(unsigned long& v)
	{
		v = (unsigned long) BYTE_SWAP_32(v);
	}

	inline void Canonical(void*& v)
	{
		v = (void*) BYTE_SWAP_32(v);
	}

	inline void Canonical(char*& v)
	{
		v = (char*) BYTE_SWAP_32(v);
	}

	inline void Canonical(WinHandle& v)
	{
		v = (WinHandle) BYTE_SWAP_32(v);
	}

	void Canonical (BreakpointType& p);
	void Canonical (CardHeaderType& p);
	void Canonical (HwrROMTokenRec& p);
	void Canonical (DatabaseHdrType& p);
	void Canonical (EventType& event);
	void Canonical (FieldAttrType& p);

#ifdef __M68EZ328HWR_H__
	void Canonical (HwrDBallEZType&);
#endif

#ifdef __M68328HWR_H__
	void Canonical (HwrDBallType&);
#endif

	void Canonical (M68KRegsType& p);
	void Canonical (PointType& p);
	void Canonical (RecordEntryType& p);
	void Canonical (RectangleType& p);
	void Canonical (RsrcEntryType& p);

	void Canonical (SlkPktHeaderType& p);

	void Canonical (SlkPktFooterType& p);

	void Canonical (SysNVParamsType& p);
	void Canonical (SysPktBodyType& p);
	void Canonical (SysPktContinueCmdType& p);
	void Canonical (SysPktDbgBreakToggleCmdType& p);
	void Canonical (SysPktFindCmdType& p);
	void Canonical (SysPktFindRspType& p);
	void Canonical (SysPktGetBreakpointsCmdType& p);
	void Canonical (SysPktGetTrapBreaksCmdType& p);
	void Canonical (SysPktGetTrapConditionsCmdType& p);
	void Canonical (SysPktReadMemCmdType& p);
	void Canonical (SysPktReadRegsCmdType& p);
	void Canonical (SysPktRPCType& p);
	void Canonical (SysPktRtnNameCmdType& p);
	void Canonical (SysPktRtnNameRspType& p);
	void Canonical (SysPktStateCmdType& p);
	void Canonical (SysPktStateRspType& p);
	void Canonical (SysPktWriteRegsCmdType& p);
	void Canonical (SysPktWriteMemCmdType& p);
	void Canonical (SysPktWriteMemRspType& p);

	void Canonical (SysPktRemoteEvtCmdType& packet);

	void Canonical (DmOpenInfoType& p);

	void Canonical(SysAppInfoType& p);


	void Canonical(MemHeapHeaderType& p);
	void Canonical(Mem1HeapHeaderType& p);
	void Canonical(Mem2HeapHeaderType& p);

	void Canonical(MemChunkHeaderType& p);
	void Canonical(Mem1ChunkHeaderType& p);

	void Canonical(MemMstrPtrTableType& p);
	void Canonical(Mem1MstrPtrTableType& p);

	void Canonical(DmAccessType& p);

	void Canonical(DatabaseDirType& p);

	void Canonical(RecordListType& p);


	void Canonical(DmSearchStateType& p);

	void Canonical(char**& p);

	void Canonical (SysPktFlashWriteType& p);

	void Canonical (sysPktExecFlashType& p);

	void Canonical (SysPktChecksumType& p);

	void Canonical (SysPktCommCmdType& p);

	void Canonical (SysPktCommRspType& p);

	void Canonical (SysPktRemoteMsgCmdType& p);

	void Canonical (SysPktWriteRegsRspType& p);

	void Canonical (SysPktSetBreakpointsCmdType& p);

	void Canonical (SysPktSetBreakpointsRspType& p);

	void Canonical (SysPktSetTrapConditionsRspType& p);

	void Canonical (SysPktSetTrapConditionsCmdType& p);

	void Canonical (SysPktSetTrapBreaksRspType& p);

	void Canonical (SysPktSetTrapBreaksCmdType& p);

	void Canonical(SysPktDbgBreakToggleRspType& p);

	void Canonical(SysLibTblEntryType& p);

#ifndef BUILDING_EMULATOR
	void Canonical(AlmEntryType& p);
	void Canonical(SysKernelInfoType& p);

// ======================================================
// The following are only defined when building the PalmOS
//  Emulator
// =====================================================
#else
	void Canonical (PenBtnInfoType& p);
	void Canonical (Point& p);
	void Canonical (SysPktGetBreakpointsRspType2& p);
	void Canonical (SysPktGetTrapBreaksRspType2& p);
	void Canonical (SysPktGetTrapConditionsRspType2& p);
	void Canonical (SysPktReadRegsRspType2& p);
	void Canonical (SysPktSetBreakpointsCmdType2& p);
	void Canonical (SysPktSetTrapBreaksCmdType2& p);
	void Canonical (SysPktSetTrapConditionsCmdType2& p);
	
  #if defined (WORDSWAP_MEMORY)

	  inline void ByteswapWords (void* start, uae_u32 length)
	  {
		  for (uae_u32 ii = 0; ii < length / 2; ++ii)
		  {
			  uae_u16*	p = ((uae_u16*) start) + ii;
			  uae_u8*		bp = (uae_u8*) p;
			  *p = (*bp << 8) | *(bp + 1);
		  }
	  }

  #else

	  inline void ByteswapWords (void*, uae_u32)
	  {
	  }

  #endif	// WORDSWAP_MEMORY

#endif //BUILDING_EMULATOR


