/*********************************************************************
 *
 * Copyright (c) 1998
 * 3Com/Palm Computing Division.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *		This product includes software developed by 3Com and its
 *		contributors.
 *
 * 4. Neither 3Com nor the names of its contributors may be used to
 *    endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE 3COM AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 3COM OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 *-------------------------------------------------------------------
 * FileName:
 *	SerLibSockets.h
 * 
 * Description:
 *
 *	This header file defines the routines used by SerTransport.cpp
 *	when the serial transport mode is set as tptSockets (as defined in
 *	SerTransport.h). 
 *
 *	The SerTransport routines are used in place of the SerXXX routines
 *	when running in a host application like PalmDebugger or PalmOSEmulator.
 *	These routines allow us to generically replace the serial comm
 *	with an alternative comm.  
 *
 *	The SerialLinkMgr.c source file contains an #ifdef near the top
 *	that remaps all the SerXXX routines to TptXXX routines. Each of
 *	the TptXXX routines contains run-time checks that dispatch to
 *	the appropriate low level comm handler routine depending on the
 *	current transport setting (stored in a global and setup by TptSetType()
 *
 * History:
 * 	21-Nov-1998  - Created 
 *
 ****************************************************************/

#pragma once


#ifdef __cplusplus
extern "C" {
#endif

	long	SerSockStart				(bool connecting);
	long	SerSockStop					(void);
	bool	SerSockConnectPending		(void);
	long	SerSockAcceptConnection		(void);
	bool	SerSockIsConnected			(void);
	long	SerSockSend					(const void* buffer, long amountToWrite,
										  short* errP);
	long	SerSockReceiveWait			(unsigned long bytes, long timeout);
	long	SerSockReceiveFlush			(long timeout);
	long	SerSockReceivePartial		(void* buffer, unsigned long count, 
										  long timeout, short* errP);
	long	SerSockReceive				(void* buffer, unsigned long count, 
										  long timeout, short* errP);

#ifdef __cplusplus
}
#endif
