/*********************************************************************
 *
 * Copyright (c) 1998-1999
 * 3Com/Palm Computing Division.  All rights reserved.
 *
 * Portions copyright (c) 1995-1998
 * Bernd Schmidt.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *		This product includes software developed by 3Com and its
 *		contributors.
 *
 * 4. Neither 3Com nor the names of its contributors may be used to
 *    endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY 3COM AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 3COM OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 *******************************************************************/

 /* 
  * UAE - The Un*x Amiga Emulator
  * 
  * Memory access functions
  *
  * Copyright 1996 Bernd Schmidt
  */

#ifdef __cplusplus
extern "C" {
#endif

#include "Switches.h"

static __inline__ uae_u32 do_get_mem_long(void* a)
{
#if WORDSWAP_MEMORY
	uae_u32	b = *(uae_u32*) a;
	return (b >> 16) | (b << 16);
//	return	(((uae_u32) *(((uae_u16*) a) + 0)) << 16) |
//			(((uae_u32) *(((uae_u16*) a) + 1)));
#else
	return *(uae_u32*) a;
#endif
}


static __inline__ uae_u16 do_get_mem_word(void* a)
{
	return *(uae_u16*) a;
}


static __inline__ uae_u8 do_get_mem_byte(void* a)
{
#if WORDSWAP_MEMORY
	return *(uae_u8*) ((long) a ^ 1);
#else
	return *(uae_u8*) a;
#endif
}


static __inline__ void do_put_mem_long(void* a, uae_u32 v)
{
#if WORDSWAP_MEMORY
	*(((uae_u16*) a) + 0) = (uae_u16) (v >> 16);
	*(((uae_u16*) a) + 1) = (uae_u16) (v);
#else
	*(uae_u32*) a = v;
#endif
}


static __inline__ void do_put_mem_word(void* a, uae_u16 v)
{
	*(uae_u16*) a = v;
}


static __inline__ void do_put_mem_byte(void* a, uae_u8 v)
{
#if WORDSWAP_MEMORY
	*(uae_u8*) ((long) a ^ 1) = v;
#else
	*(uae_u8*) a = v;
#endif
}


#define call_mem_get_func(func, addr)		((*func)(addr))
#define call_mem_put_func(func, addr, v)	((*func)(addr, v))

#undef NO_INLINE_MEMORY_ACCESS
#undef MD_HAVE_MEM_1_FUNCS

#ifdef __cplusplus
}
#endif

