/* ================================================================================== */
/* Copyright (c) 1998-1999 3Com Corporation or its subsidiaries. All rights reserved. */
/* ================================================================================== */

#ifndef __FLTK_LCDWINDOW_H__
#define __FLTK_LCDWINDOW_H__

#include "omnithread.h"
#include <FL/Fl_Double_Window.H>

#include "Document.h"

class Dynamic_Pixmap;
class MainWindow;

class LCD_window : public Fl_Double_Window
{

  private:
  
    // -----------------------------------------------------------------------------
    // constants
    // -----------------------------------------------------------------------------

    enum {
        lcdWidth = 160,
        lcdHeight = 160,
        silkScreenHeight = 60
    };

    // bit to indicate redraw is for LCD only
    enum {
        LCD_DAMAGE_BIT = 0x10
    };

  public:

    // -----------------------------------------------------------------------------
    // constructor / destructor
    // -----------------------------------------------------------------------------

    LCD_window( MainWindow *parent );
    virtual ~LCD_window( void );

    // -----------------------------------------------------------------------------
    // public methods
    // -----------------------------------------------------------------------------
  
    void damageLcdArea() { damage( LCD_DAMAGE_BIT, _lcdX, _lcdY, lcdWidth*_scale, lcdHeight*_scale ); }
    void setDocument( Document* doc ) { _doc = doc; }
    void loadSkinFile( const char* fname );
    void setOn( bool on );
    bool isOn() { return _lcdOn; }

  private:

    // -----------------------------------------------------------------------------
    // Fl_Double_Window overrides
    // -----------------------------------------------------------------------------

    virtual void draw();
    virtual int handle(int event);

    // -----------------------------------------------------------------------------
    // private methods
    // -----------------------------------------------------------------------------

    int updateBuffer( int );
    void bound( int& x, int& y, int bx, int xy, int bw, int bh );
    bool ptInRect( int x, int y, int rx, int ry, int rh, int rw );
    void initialize();
    void resetToDefault();

    // -----------------------------------------------------------------------------
    // private data members
    // -----------------------------------------------------------------------------

    // Pixmap which holds the device case image
    Dynamic_Pixmap *_case;

    // backing store buffer for LCD
    unsigned char *_buffer;

    // protects access to the buffer
    omni_mutex _bufferLock;

    // true if in drag sequence in LCD
    bool _penTracking;

    // current button we are tracking for a press or None if not
    int _currentButton;

    // parent window
    MainWindow *_parent;
    
    // the current document (data)
    Document *_doc;

    // true if the LCD display is ON
    bool _lcdOn;
    
    // -----------------------------------------------------------------------------
    // skin parameters
    
    // Offset within this window to the top left of where to draw the LCD
    int _lcdX, _lcdY;

    // scale at which to draw LCD, 1=1x, 2=2x, etc.
    int _scale;

    // lcd backlight color
    unsigned long _blRGB;

    // override background color (non-backlight)
    unsigned long _bgRGB;
    bool _useBGColor;
    
};

#endif /* __FLTK_LCDWINDOW_H__ */
