/* ================================================================================== */
/* Copyright (c) 1998-1999 3Com Corporation or its subsidiaries. All rights reserved. */
/* ================================================================================== */

#include "EmulatorCommon.h"

#include "Platform.h"   // IDS_SAVE_RAM_PROMPT
#include "Strings.r.h"   // IDS_SAVE_RAM_PROMPT
#include "Gremlins.h" // Gremlins_CanNew...

#include <FL/Fl.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include <FL/fl_ask.H>
#include <FL/fl_file_chooser.H>

#include "fltk_MainWindow.h"
#include "fltk_Dialog.h"


// -----------------------------------------------------------------------------
// file private constants
// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
// Menu Items IDs
enum {
    kMenuFile = 0,
    kMenuNewEmulator,
    kMenuOpenMemory,
    kMenuCloseEmulator,
    kMenuSaveMemory,
    kMenuSaveMemoryAs,
    kMenuSaveScreen,
    kMenuLoadApplication,
    kMenuHotSync,
    kMenuResetEmulator,
    kMenuDownloadROM,
    kMenuQuit,
    kMenuNull0,
    kMenuEdit,
    kMenuCut,
    kMenuCopy,
    kMenuPaste,
    kMenuClear,
    kMenuPreferences,
    kMenuLogging,
    kMenuDebug,
    kMenuDisplayToggle,
    kMenuNull1,
    kMenuGremlins,
    kMenuGremlinsNew,
    kMenuGremlinsStep,
    kMenuGremlinsResume,
    kMenuGremlinsStop,
    kMenuNull2,
#if HAS_PROFILING
    kMenuProfile,
    kMenuProfileInit,
    kMenuProfileStart,
    kMenuProfileStop,
    kMenuProfileDump,
    kMenuNull3,
#endif
    kMenuHelp,
    kMenuAbout
};

// -----------------------------------------------------------------------------
// Menu Items IDs
Fl_Menu_Item menu_mbar[] = {
    {"&File", 0,  0, 0, FL_SUBMENU, 0, 0, 12, 0},
    {"&New", FL_ALT+'n', 0, (void*)kMenuNewEmulator, 0, 0, 0, 12, 0},
    {"&Open...", FL_ALT+'o', 0, (void*)kMenuOpenMemory, 0, 0, 0, 12, 0},
    {"&Close", FL_ALT+'w', 0, (void*)kMenuCloseEmulator, FL_MENU_DIVIDER, 0, 0, 12, 0},
    {"&Save", FL_ALT+'s', 0, (void*)kMenuSaveMemory, 0, 0, 0, 12, 0},
    {"Save As...", 0, 0, (void*)kMenuSaveMemoryAs, 0, 0, 0, 12, 0},
    {"Save S&creen...", FL_ALT+'m', 0, (void*)kMenuSaveScreen, FL_MENU_DIVIDER, 0, 0, 12, 0},
    {"&Install Application/Database...", 0, 0, (void*)kMenuLoadApplication, 0, 0, 0, 12, 0},
    {"&HotSync", FL_ALT+'h', 0, (void*)kMenuHotSync, 0, 0, 0, 12, 0},
    {"&Reset", FL_ALT+'r', 0, (void*)kMenuResetEmulator, 0, 0, 0, 12, 0},
    {"&Transfer ROM...", FL_ALT+'d', 0, (void*)kMenuDownloadROM, FL_MENU_DIVIDER, 0, 0, 12, 0},
    {"&Quit", FL_ALT+'q', 0, (void*)kMenuQuit, 0, 0, 0, 12, 0},
    {0},
    {"&Edit", 0, 0, 0, FL_SUBMENU, 0, 0, 12, 0},
    {"Cu&t", 0, 0, (void*)kMenuCut, 0, 0, 0, 12, 0},
    {"&Copy", 0, 0, (void*)kMenuCopy, 0, 0, 0, 12, 0},
    {"&Paste", 0, 0, (void*)kMenuPaste, 0, 0, 0, 12, 0},
    {"Clea&r", 0, 0, (void*)kMenuClear, FL_MENU_DIVIDER, 0, 0, 12, 0},
    {"&Preferences...", FL_ALT+'/', 0, (void*)kMenuPreferences, 0, 0, 0, 12, 0},
    {"&Logging Options...", FL_ALT+'l', 0, (void*)kMenuLogging, 0, 0, 0, 12, 0},
    {"&Debug Options...", FL_ALT+'\\', 0, (void*)kMenuDebug, FL_MENU_DIVIDER, 0, 0, 12, 0},
    {"Display To&ggle...", FL_ALT+'g', 0, (void*)kMenuDisplayToggle, 0, 0, 0, 12, 0},
    {0},
    {"&Gremlins", 0, 0, 0, FL_SUBMENU, 0, 0, 12, 0},
    {"&New...", FL_ALT+'g', 0, (void*)kMenuGremlinsNew, FL_MENU_DIVIDER, 0, 0, 12, 0},
    {"&Step", 0, 0, (void*)kMenuGremlinsStep, 0, 0, 0, 12, 0},
    {"&Resume", 0, 0, (void*)kMenuGremlinsResume, FL_MENU_DIVIDER, 0, 0, 12, 0},
    {"Sto&p", 0, 0, (void*)kMenuGremlinsStop, 0, 0, 0, 12, 0},
    {0},
#if HAS_PROFILING
    {"&Profiling", 0,  0, 0, FL_SUBMENU, 0, 0, 12, 0},
    {"&Init", , 0, (void*)kMenuProfileInit, 0, 0, 0, 12, 0},
    {"&Start", FL_ALT+'[', 0, (void*)kMenuProfileStart, 0, 0, 0, 12, 0},
    {"Sto&p", FL_ALT+']', 0, (void*)kMenuProfileStop, 0, 0, 0, 12, 0},
    {"&Dump", 0, 0, (void*)kMenuProfileDump, 0, 0, 0, 12, 0},
    {0},
#endif
    {"&Help", 0,  0, 0, FL_SUBMENU, 0, 0, 12, 0},
    {"&About", 0, 0, (void*)kMenuAbout, 0, 0, 0, 12, 0},
    {0},
    {0}
};

// -----------------------------------------------------------------------------
// file private functions
// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
// menu item de/activate helper
static void setMenuItemStatus( Fl_Menu_Item& mi, bool status )
{
    if ( status )
        mi.activate();
    else
        mi.deactivate();
}

// -----------------------------------------------------------------------------
// called whenever my data model changes. Sets the various menu items to the
// corrected de/activated state based on the doc state.
void docChangedCallback( int property, void *param )
{
    MainWindow *mw = (MainWindow*) param;
    Document *doc = (mw != NULL)? mw->_doc : NULL;

    // a closing document is as good as no document...
    if ( (doc != NULL) && (property == Document::documentClosing) )
        doc = NULL;
    
    // File Menu
    setMenuItemStatus( menu_mbar[kMenuNewEmulator], true );
    setMenuItemStatus( menu_mbar[kMenuOpenMemory], true );
    setMenuItemStatus( menu_mbar[kMenuCloseEmulator], (doc != NULL) );
    setMenuItemStatus( menu_mbar[kMenuSaveMemory], (doc != NULL) );
    setMenuItemStatus( menu_mbar[kMenuSaveMemoryAs], (doc != NULL) );
    setMenuItemStatus( menu_mbar[kMenuSaveScreen], (doc != NULL) );
    setMenuItemStatus( menu_mbar[kMenuLoadApplication], (doc != NULL && doc->LcdIsOn()) );
    setMenuItemStatus( menu_mbar[kMenuHotSync], (doc != NULL) );
    setMenuItemStatus( menu_mbar[kMenuResetEmulator], (doc != NULL) );
    setMenuItemStatus( menu_mbar[kMenuDownloadROM], true );
    setMenuItemStatus( menu_mbar[kMenuQuit], true );

    // Edit Menu
    menu_mbar[kMenuCut].deactivate();
    menu_mbar[kMenuCopy].deactivate();
    menu_mbar[kMenuPaste].deactivate();
    menu_mbar[kMenuClear].deactivate();
    setMenuItemStatus( menu_mbar[kMenuPreferences], true );
    setMenuItemStatus( menu_mbar[kMenuLogging], true );
    setMenuItemStatus( menu_mbar[kMenuDebug], true );
    setMenuItemStatus( menu_mbar[kMenuDisplayToggle], (doc != NULL) );

    // Gremlins Menu
    setMenuItemStatus( menu_mbar[kMenuGremlinsNew], (doc != NULL && Gremlins_CanNew()) );
    setMenuItemStatus( menu_mbar[kMenuGremlinsStep], (doc != NULL && Gremlins_CanStep()) );
    setMenuItemStatus( menu_mbar[kMenuGremlinsResume], (doc != NULL && Gremlins_CanResume()) );
    setMenuItemStatus( menu_mbar[kMenuGremlinsStop], (doc != NULL && Gremlins_CanStop()) );

#if HAS_PROFILING
    // Profile Menu
    setMenuItemStatus( menu_mbar[kMenuProfileInit], (!gProfilingEnabled) );
    setMenuItemStatus( menu_mbar[kMenuProfileStart], (!gProfilingEnabled) );
    setMenuItemStatus( menu_mbar[kMenuProfileStop], (gProfilingEnabled && gProfilingOn) );
    setMenuItemStatus( menu_mbar[kMenuProfileDump], (gProfilingEnabled && gProfilingOn) );
#endif

    // Help Menu
    setMenuItemStatus( menu_mbar[kMenuAbout], true );    
}

// -----------------------------------------------------------------------------
// calback for main application menu items
void menuCallback( Fl_Menu_* w, void *data )
{
    const Fl_Menu_Item *mi = ((Fl_Menu_*)w)->mvalue();
    MainWindow *mw = (MainWindow*)data;

    switch ( mi->argument() )
    {
        case kMenuNewEmulator:
            mw->doEmulatorStart();
            break;
            
        case kMenuCloseEmulator:
            mw->doEmulatorStop();
            break;
            
        case kMenuResetEmulator:
            mw->_doc->emulatorReset();
            break;
            
        case kMenuHotSync:
            mw->_doc->hotsync();
            break;
            
        case kMenuOpenMemory:
            mw->doOpenMemory();
            break;
            
        case kMenuSaveMemoryAs:
            mw->doSaveMemoryAs();
            break;
            
        case kMenuSaveScreen:
            mw->doSaveScreen();
            break;
            
        case kMenuLoadApplication:
            mw->doLoadApplication();
            break;
            
        case kMenuQuit:
            mw->doQuit();
            break;

        case kMenuPreferences:
            mw->doEditPreferences();
            break;
            
        case kMenuDebug:
            mw->doEditDebugPreferences();
            break;

        case kMenuDisplayToggle:
            mw->_lcd->setOn( !mw->_lcd->isOn() );
            break;
            
        case kMenuLogging:
            mw->doEditLoggingPreferences();
            break;
            
        case kMenuGremlinsNew:
			mw->_doc->gremlinsNew();
            break;

        case kMenuGremlinsStep:
            mw->_doc->gremlinsStep();
            break;

        case kMenuGremlinsResume:
            mw->_doc->gremlinsResume();
            break;

        case kMenuGremlinsStop:
            mw->_doc->gremlinsStop();
            break;

#if HAS_PROFILING
        case kMenuProfileInit:
            //::ProfileInit( MAXFNCALLS, MAXDEPTH );
            ::ProfileInit( 0x0C000, MAXDEPTH );
            break;

        case kMenuProfileStart:
            ::ProfileStart();
            break;

        case kMenuProfileStop:
            ::ProfileStop ();
            break;

        case cmd_Profile_Dump:
            ::ProfileDump( NULL );
            ::ProfileCleanup();
            break;
#endif

        case kMenuAbout:
            showAboutBox();
            break;

        default:
            break;
    }
}

// -----------------------------------------------------------------------------
// constructor
// -----------------------------------------------------------------------------

MainWindow::MainWindow()
        : Fl_Window( 250, 358 ), _doc( NULL )
{
    box( FL_FLAT_BOX );
    color( fl_gray_ramp( FL_NUM_GRAY-1 ) );

    resizable( NULL );
    
    _menuButton= new Fl_Menu_Button(0,0,600,400,"POSER");
    _menuButton->type( Fl_Menu_Button::POPUP3 );
    _menuButton->menu( menu_mbar );
    _menuButton->callback( (Fl_Callback*) menuCallback, this );

    _lcd = new LCD_window( this );

    // if there is a skin file path stored in the preferences, load it.
    const char* spath = gHostPrefs->getSkinFile();
    if ( spath && ( strlen( spath ) > 0 ) )
    {
        _lcd->loadSkinFile( spath );
    }
    
    end();

    // call the menu setup function
    docChangedCallback( 0, this );

	if (_sMainWindow == NULL)
		_sMainWindow = this;
}

// -----------------------------------------------------------------------------
// private methods
// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
// quit the application.
void MainWindow::doQuit()
{
    if ( _doc != NULL )
        doEmulatorStop();

	if (_sMainWindow == this)
		_sMainWindow = NULL;

    delete this;
}

// -----------------------------------------------------------------------------
// create a new data document & start the emulator
void MainWindow::doEmulatorStart()
{
    AppPreferences::Configuration cfg = gPrefs->GetLastConfiguration();

    // ask the user for doc creation parameters
    if ( !showNewEmulatorWindow( cfg ) )
        return;
    
    // if there is a running document, clear it out.
    if ( _doc )
        doEmulatorStop();

    // create the new document
    _doc = new Document( cfg );
    if ( _doc->initializedOk() )
    {
        _lcd->setDocument( _doc );
        _doc->addPropertyChangedHandler( docChangedCallback, (void*)this );
        _doc->emulatorStart();
    }
    else
    {
        delete _doc;
        _doc = NULL;
        _lcd->setDocument( NULL );
    }
}

// -----------------------------------------------------------------------------
// stop the emulator and kill the existing data document. Ask the user first.
void MainWindow::doEmulatorStop()
{
	int	action = gPrefs->GetCloseAction();

	if ( action == AppPreferences::kSaveAsk )
	{
		string saveChanges( Platform::GetString( kStr_SaveBeforeClosing ) );
		string untitled( Platform::GetString( kStr_Untitled ) );
		string app( Platform::GetString( kStr_AppName ) );

		ParamList paramList;
		paramList.push_back( string( "%AppName" ) );
		paramList.push_back( app );
		paramList.push_back( string( "%DocName" ) );
        if ( _doc->getCurrentRAMRef().IsSpecified() )
            paramList.push_back( _doc->getCurrentRAMRef().GetFileName() );
        else
            paramList.push_back( untitled );

        string msg = Errors::ReplaceParameters( saveChanges, paramList );

        int result = fl_choice( msg.c_str(),
                                Platform::GetString( kStr_Cancel ).c_str(),
                                Platform::GetString( kStr_Yes ).c_str(),
                                Platform::GetString( kStr_No ).c_str() );

		if ( result == 1 ) // yes
			action = AppPreferences::kSaveAlways;

		else if ( result == 2 ) // no
			action = AppPreferences::kSaveNever;

		else		// result == IDCANCEL
			return;
	}

    _doc->emulatorStop();

	if ( action == AppPreferences::kSaveAlways )
	{
        _doc->saveMemory();
	}

    _lcd->setDocument( NULL );
    _lcd->redraw();

    delete _doc;
    _doc = NULL;
}

// -----------------------------------------------------------------------------
// load an application file into the current running emulator
void MainWindow::doLoadApplication()
{
    // resource-ize this stuff...
    char *fname = fl_file_chooser( "Choose an Application or Database",
#if SUPPORT_CLIPPINGS
                                   "*.{prc,PRC,pdb,PDB,pqa,PQA}",
#else
                                   "*.{prc,PRC,pdb,PDB}",
#endif
                                   NULL );

    if ( fname != NULL )
    {
        FileReference fileRef( fname );
        _doc->loadApplication( fileRef );
    }
}

// -----------------------------------------------------------------------------
// save the document with a new file name
void MainWindow::doSaveMemoryAs()
{
	char fileName[PATH_MAX] = {0};
    
    char *fname = fl_file_chooser( "Save RAM file as", "*.psf", fileName );
    
    if ( fname == NULL )
    {
        return;
    }
    
    strcpy( fileName, fname );
    
	FileReference ramFileRef = FileReference( fileName );

    _doc->saveMemoryAs( ramFileRef );
}

// -----------------------------------------------------------------------------
// open a pre-existing document.
void MainWindow::doOpenMemory()
{
    // get the document name from the user
	char fileName[PATH_MAX] = {0};
    char *fname = fl_file_chooser( "Open RAM file", "*.psf", fileName );
    if ( fname == NULL )
    {
        return;
    }

    // get a FileReference to the given file
	FileReference ramFileRef = FileReference( fname );

    // Clear out othe old document
    if ( _doc )
    {
        _lcd->setDocument( NULL );
        delete _doc;
        _doc = NULL;
    }

    // create a new document & set up the pointers
    _doc = new Document( gPrefs->GetLastConfiguration() );
    if ( _doc->initializedOk() )
    {
        _lcd->setDocument( _doc );
        _doc->addPropertyChangedHandler( docChangedCallback, (void*)this );

        // go ahead and open the document
        _doc->open( ramFileRef );
    }
    else
    {
        delete _doc;
        _doc = NULL;
    }
}

// -----------------------------------------------------------------------------
// save the screen image
void MainWindow::doSaveScreen()
{
    FileReference f;
    _doc->saveScreen( f );
}

// -----------------------------------------------------------------------------
// preference editing
void MainWindow::doEditLoggingPreferences()
{
    if ( _doc != NULL )
        _doc->emulatorStop();

    if ( gHostPrefs->editLogging() )
    {
        // possibly take action based on new prefs...
        ;
    }

    if ( _doc != NULL )
        _doc->emulatorStart();
}

void MainWindow::doEditDebugPreferences()
{
    if ( _doc != NULL )
        _doc->emulatorStop();

    if ( gHostPrefs->editDebugging() )
    {
        // possibly take action based on new prefs...
        ;
    }

    if ( _doc != NULL )
        _doc->emulatorStart();
}

void MainWindow::doEditPreferences()
{
    if ( _doc != NULL )
        _doc->emulatorStop();

    // remember the current skin:
    string curskin( gHostPrefs->getSkinFile() );
    
    // let the user edit the preferences.
    if ( gHostPrefs->editPreferences() )
    {
        const char *spath = gHostPrefs->getSkinFile();
        if ( strcmp( curskin.c_str(), spath ) != 0 )
        {
            // skin has changed. notify Lcd
            _lcd->loadSkinFile( gHostPrefs->getSkinFile() );
        }
    }

    if ( _doc != NULL )
        _doc->emulatorStart();
}

MainWindow* MainWindow::_sMainWindow = NULL;
