/* ================================================================================== */
/* Copyright (c) 1998-1999 3Com Corporation or its subsidiaries. All rights reserved. */
/* ================================================================================== */

#include "EmulatorCommon.h"

#include "Platform.h"    // gPseudoTickcount

#include <FL/Fl.H>
#include <FL/Fl_Window.H>

#include "ResStrings.h"
#include "fltk_MainWindow.h"
#include "fltk_main.h"
#include "Preferences.h"

// -----------------------------------------------------------------------------
// execute the given FN within the context of the UI thread. This function
// (fltkExecuteUI) will be called from a non-UI thread. Currently there is
// only support for ONE pending function

static void (*sUIFunc)( void ) = NULL;
static omni_mutex sUIFuncMutex;
static omni_thread *sMainThread;

void fltkExecuteUI( void (*fn)( void ) )
{
    sUIFuncMutex.lock();
    sUIFunc = fn;
    sUIFuncMutex.unlock();
}

bool currentlyUIThread()
{
    return omni_thread::self() == sMainThread;
}

int main(int argc, char **argv)
{
    // hold onto the main thread
    sMainThread = omni_thread::self();
    
    _ResStrTableInit();
    gHostPrefs->loadFromFile();
    
    MainWindow *mainWin = new MainWindow;
        
    Fl::visual(FL_DOUBLE|FL_RGB);
    mainWin->show(argc, argv);
        
    while ( Fl::wait()  )
    {
        gPseudoTickcount++;
        sUIFuncMutex.lock();
        if ( sUIFunc != NULL )
        {
            (*sUIFunc)();
            sUIFunc = NULL;
        }
        sUIFuncMutex.unlock();
    }

    gHostPrefs->storeToFile();
    return 0;
}
